#!/usr/bin/env python
# -*- coding: utf-8 -*-


# subaloud.py - SubAloud + Milena gui
# Copyright (C) Bohdan R. Rau 2013 <ethanak@polip.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write see:
#               <http://www.gnu.org/licenses/>.


import sys,os,re,signal,pickle,socket,shlex,urllib
from gi.repository import Gtk,GLib

_ISAVI=re.compile(r'\.(avi|mpeg|mkv|rmvb|mp4)$',re.I)
_NOEXT=re.compile(r'^(.*)(\.[^.]+)$',re.I)
_FINES=['.asub','.sub','.srt','.ass','.txt']
_EQLABS=['31.25','62.5','125','250','500','1k','2k','4k','8k','16k']

def _get_entry(self):
    children=self.get_children()
    for child in children:
        if child.__class__.__name__ == 'Entry':
            return child
    return None

def _get_array(self):
    ls=self.get_model()
    itr=ls.get_iter_first()
    a=[]
    while itr:
        a.append(ls.get_value(itr,0))
        itr = ls.iter_next(itr)
    return a

def _combo_box_len(self):
    return self.get_model().iter_n_children(None)

Gtk.ComboBoxText.get_entry=_get_entry
Gtk.ComboBoxText.get_array=_get_array
Gtk.ComboBoxText.__len__=_combo_box_len





class gui(Gtk.Window):
    _version='0.1.4'
    def __init__(self):
        Gtk.Window.__init__(self)
        self.rm=Gtk.RecentManager.get_default()
        d=os.path.join(os.environ['HOME'],'.local/share')
        if os.path.isdir(d):
            self._prefs_path=os.path.join(d,'subaloud_mg')
        else:
            self._prefs_path=os.path.join(os.environ['HOME'],'.subaloud_mg')
        try:
            self._prefs=pickle.load(open(self._prefs_path))
        except:
            self._prefs={
                'voice_name':'Milena',
                'tempo_normal':1.3,
                'tempo_max':1.6
            }
        self._prefs_changed=False
        self.movie=None
        self.last_movie=None
        self.mplayer_pid=0
        self.temp_subtitle=None
        self.current_decisec=0
        self.pause_mode=False
        self._make_frame()
        
    def alert(self,txt,iserror=False):
        dialog=Gtk.MessageDialog(self,3,Gtk.MessageType.ERROR if iserror else Gtk.MessageType.INFO,Gtk.ButtonsType.OK,txt)
        dialog.run()
        dialog.destroy()

    def _setpref(self,name,value):
        if self._prefs.get(name) != value:
            self._prefs[name]=value
            self._prefs_changed=True

    def _storepref(self):
        if self._prefs_changed:
            pickle.dump(self._prefs,open(self._prefs_path,'w'))
            self._prefs_changed=False

    def exit(self,*args):
        from ctypes import CDLL
        if self.mplayer_pid:
            os.kill(self.mplayer_pid,signal.SIGINT)
        if self.temp_subtitle:
            try:
                os.remove(self.temp_subtitle)
            except:
                pass
        CDLL("libc.so.6").exit(0)

    def loop(self):
        Gtk.main()

    
    def getsubnames(self,filename,firstonly=False):
        r=_NOEXT.match(filename)
        if not r:
            return None
        rc=[]
        p=r.group(1)
        for fin in _FINES:
            if os.path.isfile(p+fin):
                if firstonly:
                    return True
                rc.append(p+fin)
        fname=os.path.join(os.environ['HOME'],'tmp/milena_movie',os.path.basename(p),'dumpsub.txt')
        if os.path.isfile(fname):
            if firstonly:
                return True
            rc.append(fname)
        if firstonly:
            return False
        return rc;
            
    
    def myfilter(self,filterinfo,data):
        path=filterinfo.filename
        uri=filterinfo.uri
        if not uri.startswith('file:///'):
            return False
        r=_ISAVI.search(path)
        if not r:
            return False
        return self.getsubnames(path,True)
    
    def selectMovie(self):
        dialog=Gtk.FileChooserDialog('Otwórz film',self,Gtk.FileChooserAction.OPEN,
                                     (Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OK,Gtk.ResponseType.OK))
        dialog.set_local_only(True)
        if self.movie:
            dialog.set_current_folder(os.path.dirname(self.movie))
        elif os.getcwd() == os.environ['HOME']:
            fs=os.path.join(os.environ['HOME'],'Wideo')
            if os.path.isdir(fs):
                dialog.set_current_folder(fs)
        f1=Gtk.FileFilter()
        f1.set_name('Wszystkie filmy')
        f1.add_pattern('*.avi')
        f1.add_pattern('*.mkv')
        f1.add_pattern('*.mpeg')
        f1.add_pattern('*.rmvb')
        f1.add_pattern('*.mp4')
        dialog.add_filter(f1)
        f2=Gtk.FileFilter()
        f2.set_name('Filmy z napisami')
        f2.add_custom(Gtk.FileFilterFlags.URI | Gtk.FileFilterFlags.FILENAME,self.myfilter,None)
        dialog.add_filter(f2)
        f3=Gtk.FileFilter()
        f3.set_name('Wszystkie pliki')
        f3.add_pattern('*')
        dialog.set_filter(f1)
        #import signal
        #signal.signal(signal.SIGINT, signal.SIG_DFL)

        rc=dialog.run()
        rs=None
        if rc == Gtk.ResponseType.OK:
            rs=dialog.get_filename()
        dialog.destroy()
        return rs
    
    def selectDic(self):
        dialog=Gtk.FileChooserDialog('Wybierz plik słownika',self,Gtk.FileChooserAction.OPEN,
                                     (Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OK,Gtk.ResponseType.OK))
        dialog.set_local_only(True)
        if self.movie:
            dialog.set_current_folder(os.path.dirname(self.movie))
        elif os.getcwd() == os.environ['HOME']:
            fs=os.path.join(os.environ['HOME'],'Wideo')
            if os.path.isdir(fs):
                dialog.set_current_folder(fs)
        f1=Gtk.FileFilter()
        f1.set_name('Słowniki')
        f1.add_pattern('*.dic')
        dialog.add_filter(f1)
        f3=Gtk.FileFilter()
        f3.set_name('Wszystkie pliki')
        f3.add_pattern('*')
        dialog.add_filter(f3)
        dialog.set_filter(f1)
        #import signal
        #signal.signal(signal.SIGINT, signal.SIG_DFL)

        rc=dialog.run()
        rs=None
        if rc == Gtk.ResponseType.OK:
            rs=dialog.get_filename()
        dialog.destroy()
        return rs

    def selectSub(self):
        dialog=Gtk.FileChooserDialog('Wybierz plik napisów',self,Gtk.FileChooserAction.OPEN,
                                     (Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OK,Gtk.ResponseType.OK))
        dialog.set_local_only(True)
        if self.movie:
            dialog.set_current_folder(os.path.dirname(self.movie))
        elif os.getcwd() == os.environ['HOME']:
            fs=os.path.join(os.environ['HOME'],'Wideo')
            if os.path.isdir(fs):
                dialog.set_current_folder(fs)
        f1=Gtk.FileFilter()
        f1.set_name('Napisy')
        f1.add_pattern('*.asub')
        f1.add_pattern('*.sub')
        f1.add_pattern('*.srt')
        f1.add_pattern('*.txt')
        f1.add_pattern('*.ass')
        f1.add_pattern('*.ssa')
        dialog.add_filter(f1)
        f3=Gtk.FileFilter()
        f3.set_name('Wszystkie pliki')
        f3.add_pattern('*')
        dialog.add_filter(f3)
        dialog.set_filter(f1)
        #import signal
        #signal.signal(signal.SIGINT, signal.SIG_DFL)

        rc=dialog.run()
        rs=None
        if rc == Gtk.ResponseType.OK:
            rs=dialog.get_filename()
        dialog.destroy()
        return rs

    def selectSaveSub(self):
        dialog=Gtk.FileChooserDialog('Wybierz plik napisów',self,Gtk.FileChooserAction.SAVE,
                                     (Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OK,Gtk.ResponseType.OK))
        dialog.set_local_only(True)
        if self.movie:
            r=_NOEXT.match(self.movie)
            if not r:
                r=self.movie+'.txt'
            else:
                r=r.group(1)+'.txt'
            dialog.set_current_folder(os.path.dirname(self.movie))
            dialog.set_current_name(os.path.basename(r))
        dialog.set_create_folders(True)
        f1=Gtk.FileFilter()
        f1.set_name('Napisy')
        f1.add_pattern('*.asub')
        f1.add_pattern('*.sub')
        f1.add_pattern('*.srt')
        f1.add_pattern('*.txt')
        dialog.add_filter(f1)
        f3=Gtk.FileFilter()
        f3.set_name('Wszystkie pliki')
        f3.add_pattern('*')
        dialog.set_filter(f1)
        #import signal
        #signal.signal(signal.SIGINT, signal.SIG_DFL)

        rc=dialog.run()
        rs=None
        if rc == Gtk.ResponseType.OK:
            rs=dialog.get_filename()
        dialog.destroy()
        return rs


    def add_recent(self,path):
        path=os.path.abspath(path)
        if isinstance(path, unicode):
            path = path.encode('utf8')
        uri='file://' + urllib.pathname2url(path)
        self.rm.add_item(uri)

    def set_movie(self,path):
        self.movie=path
        self.add_recent(path)
        self.widget_moviename.set_text(path)
        self.subnames=self.getsubnames(path)
        self.subtitle_aloud.remove_all()
        self.subtitle_displ.remove_all()
        self.subtitle_aloud_entry.set_text('')
        self.subtitle_displ.append_text('Nie wyświetlaj napisów')
        for a in self.subnames:
            self.subtitle_aloud.append_text(a)
            if a.find('/tmp/milena_movie/') < 0:
                self.subtitle_displ.append_text(a)
        if len(self.subnames) > 0:
            self.subtitle_aloud.set_active(0)
        self.subtitle_displ.set_active(0)
    
    def _open_movie(self,*args):
        m=self.selectMovie()
        if m:
            self.set_movie(m)
            
    def _open_dic(self,widget,entry):
        s=self.selectDic()
        if s:
            entry.set_text(s)
    
    def _open_subtitles(self,widget,combo):
        s=self.selectSub()
        if s:
            combo.append_text(s)
            if s.find('/tmp/milena_movie/') < 0:
                self.add_recent(s)
            combo.set_active(combo.get_model().iter_n_children(None)-1)
    
    def _napiprojekt(self, *args):
        import NapiProjekt
        try:
            np=NapiProjekt.NapiProjekt(self.movie,False)
            sub=np.downloadSubtitles()
            if not sub:
                self.alert("Nie znaleziono napisów")
                return
        except:
            import traceback
            self.alert(traceback.format_exc(),True)
            return
        fname=self.selectSaveSub()
        if fname:
            try:
                open(fname,'wb').write(sub)
            except:
                import traceback
                self.alert(traceback.format_exc(),True)
                return
            self.subtitle_aloud.append_text(fname)
            self.subtitle_aloud.set_active(len(self.subtitle_aloud)-1)
            
    
    def _napiprojekt_old(self,*args):
        import pynapi
        try:
            digest=pynapi.calculate_digest(self.movie)
            sub = pynapi.get_subtitle(digest, 'PL')
        except:
            import traceback
            self.alert(traceback.format_exc(),True)
            return
        fname=self.selectSaveSub()
        if fname:
            try:
                open(fname,'wb').write(sub)
            except:
                import traceback
                self.alert(traceback.format_exc(),True)
                return
            self.subtitle_aloud.append_text(fname)
            self.subtitle_aloud.set_active(len(self.subtitle_aloud)-1)
        
                
        
    def _send_subaloud(self,cmd):
        sname="\0SubAloud.ctrl"
        try:
            socky=socket.socket(socket.AF_UNIX,socket.SOCK_STREAM)
            socky.connect(sname)
            socky.sendall(cmd)
            return socky.recv(4096)
        except:
            return None

    def _setvol(self,widget,data):
        value=int(widget.get_value())
        cmd='v%d\n' % value
        self._send_subaloud(cmd)
        
    def _resync(self,widget,data):
        self._send_subaloud('s\n')

    def _setdel(self,widget,data):
        value=int(widget.get_value()*10)
        cmd='d%d\n' % value
        self._send_subaloud(cmd)

    def _wait_for_mplayer(self,*args):
        try:
            res=os.waitpid(self.mplayer_pid,os.WNOHANG)
        except:
            return True
        if not res or not res[0]:
            return True
        self.mplayer_pid=None
        if self.temp_subtitle:
            os.remove(self.temp_subtitle)
            self.temp_subtitle=None
        if self.pause_mode:
            self.pause_mode=0
            self.pilot.hide()
            self.mainbox.show_all()
            self._equali_onoff()
            self.resize(10,10)
            return False
            
        self.exit(0)
    
    def _pause(self,*args):
        if not self.mplayer_pid:
            return
        v=self._send_subaloud('p\n')
        if not v.startswith('+p='):
            return
        self.current_decisec=int(v[3:])
        self.pause_mode=True
        os.kill(self.mplayer_pid,signal.SIGINT)
    
    def _getextras(self,extras):
        arr=self.mpl_extras.get_array()
        if extras:
            found=-1
            for i,n in enumerate(arr):
                if extras == n:
                    found=i
                    break
            if found > 0:
                self.mpl_extras.remove(found)
                self.mpl_extras.insert_text(0,extras)
            else:
                self.mpl_extras.insert_text(0,extras)
            arr=self.mpl_extras.get_array()
            self.mpl_extras_dele.set_sensitive(True)
        return '\n'.join(arr)
                
    
    def _start_movie(self,*args):
        if not self.movie:
            return
        if self.subtitle_aloud.get_model().iter_n_children(None) == 0:
            self.alert('Brak napisów do czytania',True)
            return
        extras=None
        s=self.mpl_extras.get_entry().get_text().strip()
        if s:
            try:
                extras=shlex.split(s)
            except:
                self.alert('Błąd parsowania opcji dodatkowych:\n%s' % sys.exc_info()[1],True)
                return
        eqv=(':'.join('%.1f' % x.get_value() for x in self.equidgets))
        self._setpref('voice_name',self._voice_selector.get_active_text())
        self._setpref('tempo_normal',self._speed_widget.get_value())
        self._setpref('tempo_max',self._mspeed_widget.get_value())
        self._setpref('delay',self._delay_widget.get_value())
        self._setpref('volnorm',self.mpl_volnorm.get_active())
        self._setpref('fullscreen',self.fullscreen_cb.get_active())
        self._setpref('volctl',self.mpl_volcontrol.get_active())
        self._setpref('equctl',self.mpl_equali.get_active())
        self._setpref('equval',eqv)
        self._setpref('diction',self.subtitle_dic.get_text())
        self._setpref('extras',self._getextras(s))
        
        self._storepref()
        subname=self.subtitle_aloud.get_active_text()
        dicname=self.subtitle_dic.get_text().strip()
        inilang=[]
        inithem=[]
        if not dicname:
            r=_NOEXT.match(subname)
            if r:
                dicname=r.group(1)+".dic"
                if not os.path.isfile(dicname):
                    dicname=None
        if dicname:
            f=open(dicname)
            for i in range(1,4):
                line=f.readline()
                if not line:
                    break
                line=line.strip()
                if not line:
                    continue
                if not line.startswith('//'):
                    break
                line=re.sub(r'\s+','',line[2:])
                #print line
                if line.startswith('#LANGS:'):
                    for a in line[7:].split(','):
                        inilang.append(a)
                    continue
                if line.startswith('#THEMES:'):
                    for a in line[8:].split(','):
                        inithem.append(a)
                    continue
            f.close()
                        
        if self.autoconnect_cb.get_active():
            vd=os.path.join(os.environ['HOME'],'tmp')
            if not os.path.isdir(vd):
                try:
                    os.mkdir(vd)
                except:
                    self.alert("Nie mogę utworzyć katalogu ~/tmp",True)
                    return
            mname=os.path.join(vd,os.path.basename(subname)+'.tmp')
            import subconvert
            cv=subconvert.SubConverter()
            try:
                rc=cv.simplify(subname,mname)
            except:
                import traceback
                traceback.print_exc()
                rc=traceback.format_exc()
            if rc:
                self.alert(rc,True)
                return
            self.temp_subtitle=mname
            self.subname=mname
        if self.movie != self.last_movie:
            self.current_decisec=0
            self.last_movie=self.movie
        if self.current_decisec == 0:
            self._delscale.set_value(self._prefs['delay'])


        cmd=["subaloud","-M","milena","-V",self._voice_selector.get_active_text(),
             "-r",str(self._speed_widget.get_value()),
             "-R",str(self._mspeed_widget.get_value()),
             ]
        dly=int(self._delscale.get_value() * 10)        
        if dly:
            cmd.append('-D')
            cmd.append(str(dly))
        vly=int(self._volscale.get_value())
        if vly:
            cmd.append('-A')
            cmd.append(str(vly))
        ls=[]
        for n in self.languages:
            if n[2].get_active() or n[0] in inilang:
                ls.append(n[0])
        if ls:
            cmd.append('--mod-lang=%s' % ','.join(ls))
        ls=[]
        for n in self.themes:
            if n[1].get_active() or n[0] in inithem:
                ls.append(n[0])
        if ls:
            cmd.append('--mod-theme=%s' % ','.join(ls))
        if dicname:
            cmd.append('--mod-dict=%s' % dicname)
        cmd.append('-s')
        cmd.append(subname)
        cmd.append('-m')
        cmd.append(self.movie)
        cmd.append('--')



        if self.subtitle_displ.get_active() < 1:
            cmd.append('-nosub')
            cmd.append('-noautosub')
        else:
            cmd.append('-sub')
            cmd.append(self.subtitle_displ.get_active_text())
            cmd.append('-subcp')
            cmd.append('enca:pl:cp1250')
        n=self.mpl_volcontrol.get_active()
        if n > 0:
            cmd.append('-softvol')
            if n > 1:
                cmd.append('-softvol-max')
                cmd.append(('300','1000','3000')[n-2])
            
        if self.current_decisec > 50:
            cmd.append('-ss')
            cmd.append('%.d' % int(self.current_decisec/10-5))
        af=[]
        if self.mpl_volnorm.get_active():
            af.append('volnorm=2')
        if self.mpl_equali.get_active():
            af.append('equalizer=' + eqv)
        if af:
            cmd.append('-af')
            cmd.append(','.join(af))
        
        if self.deinterlace_cb.get_active():
            cmd.append('-vf')
            cmd.append('kerndeint')
        
        if self.fullscreen_cb.get_active():
            cmd.append('-fs')
        
        if extras:
            cmd += extras
        
        
        pid=os.fork()
        if pid < 0:
            return
        if pid == 0:
            os.execv("/usr/bin/subaloud",cmd)
            sys.exit(1)
            
        self.mplayer_pid=pid
        GLib.timeout_add(300,self._wait_for_mplayer,None)

        self.mainbox.hide()
        self.pilot.show_all()
        x,y=self.get_size()
        self.resize(x,10)

    def _equali_onoff(self,*args):
        if self.mpl_equali.get_active():
            self.frame_equali.show_all()
        else:
            self.frame_equali.hide()
        x,y=self.get_size()
        self.resize(x,10)
        
    def _clentry(self,widget,*args):
        widget.set_text('')
        
    def _prextras(self,*args):
        txt=self.mpl_extras.get_entry().get_text()
        tn=False
        for ar in self.mpl_extras.get_array():
            if ar == txt:
                tn=True
                break
        self.mpl_extras_dele.set_sensitive(tn)
    
    def _delextras(self,*args):
        txt=self.mpl_extras.get_entry().get_text()
        tn=-1
        for i,ar in enumerate(self.mpl_extras.get_array()):
            if ar == txt:
                tn=i
                break
        if tn < 0:
            return
        self.mpl_extras.remove(tn)
        self.mpl_extras.get_entry().set_text('')
        
    def _make_frame(self):
        signal.signal(signal.SIGINT, signal.SIG_DFL)
        self.milena_dir="/usr/share/milena"
        self.libdir=os.path.dirname(__file__)
        self.languages=[
            ['en','Angielski',None],
            ['fr',"Francuski",None],
            ['es',"Hiszpański",None],
            ['it',"Włoski",None],
            ['se',"Szwedzki",None],
            ['ru',"Rosyjski",None],
            ['de',"Niemiecki",None],
            ['hu',"Węgierski",None],
            ['ro',"Rumuński",None],
            ['pt',"Portugalski",None]]
        self.langwidgets={}

        self.cid=self.connect("destroy",self.exit)
        self.set_title('SubAloud '+self._version)
        f=os.path.join(self.libdir,'subaloudmil.png')
        self.set_icon_from_file(f)
        frame=Gtk.VBox(0,2)
        self.add(frame)
        
        self.mainbox=Gtk.VBox(0,2)
        frame.add(self.mainbox)
        
        lf=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf,0,0,0)
        
        langframe=Gtk.Frame.new('Język oryginału')
        lf.pack_start(langframe,0,0,0)
        tab=Gtk.Grid()
        langframe.add(tab)
        for i,n in enumerate(self.languages):
            x=i / 2;
            y=i & 1
            cb=Gtk.CheckButton.new_with_label(n[1])
            n[2]=cb;
            self.langwidgets[n[0]]=cb;
            tab.attach(cb,x,y,1,1)
        themelist=[]
        a=os.listdir(self.milena_dir)
        for t in a:
            r=re.match('pl_(.*)_theme\.dat$',t)
            if r:
                themelist.append([t,r.group(1),None,None])
        for a in themelist:
            try:
                f=open(os.path.join(self.milena_dir,a[0])).readline()
                if f.startswith('//'):
                    f=f[2:]
                else:
                    continue
                f=f.strip().decode('iso-8859-2')
                a[2]=f
            except:
                pass
        themelist.sort()
        themframe=Gtk.Frame.new('Tematy')
        lf.pack_start(themframe,0,0,0)
        tab=Gtk.Grid()
        themframe.add(tab)
        self.themes=[]
        self.themewidgets={}
        n=0
        for a in themelist:
            if not a[2]:
                continue
            cb=Gtk.CheckButton.new_with_label(a[1])
            cb.set_tooltip_text(a[2])
            tab.attach(cb,n /2, n &1, 1, 1);
            self.themes.append([a[1],cb])
            self.themewidgets[a[1]]=cb
            n += 1
        
        self.vlist=['Milena']
        nvo=0
        try:
            f=open(os.path.join(os.environ['HOME'],'.ivolektor.cfg'))
            for line in f:
                r=re.match(r'\s*\[(.*)\]',line)
                if r:
                    if str(r.group(1)) == str(self._prefs['voice_name']):
                        nvo=len(self.vlist)
                    self.vlist.append(r.group(1))
        except:
            pass
        lf1=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf1,True,True,0)
        lb=Gtk.Label('Głos')
        lf1.pack_start(lb,0,0,0)
        self._voice_selector=Gtk.ComboBoxText()
        for a in self.vlist:
            self._voice_selector.append_text(a)
        self._voice_selector.set_active(nvo)
        lf1.pack_start(self._voice_selector,0,0,0)
        lb=Gtk.Label('Prędkość')
        lf1.pack_start(lb,0,0,0)
        self._speed_widget=Gtk.SpinButton.new_with_range(1.0,2.0,0.05)
        self._speed_widget.set_digits(2)
        self._speed_widget.set_value(self._prefs['tempo_normal'])
        lf1.pack_start(self._speed_widget,0,0,0)
        lb=Gtk.Label('Max.')
        lf1.pack_start(lb,0,0,0)
        self._mspeed_widget=Gtk.SpinButton.new_with_range(1.0,2.0,0.05)
        self._mspeed_widget.set_digits(2)
        self._mspeed_widget.set_value(self._prefs['tempo_max'])
        lf1.pack_start(self._mspeed_widget,0,0,0)
        
        lb=Gtk.Label('Opóźnienie')
        lf1.pack_start(lb,0,0,0)
        
        self._delay_widget=Gtk.SpinButton.new_with_range(-1.5,1.5,0.1)
        self._delay_widget.set_digits(1)
        self._delay_widget.set_value(self._prefs.get('delay',0.6))
        lf1.pack_start(self._delay_widget,0,0,0)
        
        
        lf2=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf2,True,True,0)
        lb=Gtk.Label('Film')
        lb.set_alignment(1.0,0.5)
        lb.set_size_request(100,0)
        lf2.pack_start(lb,0,0,0)
        self.widget_moviename=Gtk.Entry()
        self.widget_moviename.set_editable(False)
        lf2.pack_start(self.widget_moviename,True,True,0)
        mvbutton=Gtk.Button.new_from_stock(Gtk.STOCK_OPEN)
        mvbutton.set_size_request(80,0)
        lf2.pack_start(mvbutton,0,0,0)
        mvbutton.connect('clicked',self._open_movie,None)
        
        lf3=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf3,True,True,0)
        lb=Gtk.Label('Czytane')
        lb.set_alignment(1.0,0.5)
        lb.set_size_request(100,0)
        lf3.pack_start(lb,0,0,0)
        
        self.subtitle_aloud=Gtk.ComboBoxText.new_with_entry()
        self.subtitle_aloud_entry=self.subtitle_aloud.get_children()[0]
        self.subtitle_aloud_entry.set_editable(False)
        lf3.pack_start(self.subtitle_aloud,True,True,0)
        albutton=Gtk.Button.new_from_stock(Gtk.STOCK_OPEN)
        albutton.set_size_request(80,0)
        lf3.pack_start(albutton,0,0,0)
        albutton.connect('clicked',self._open_subtitles,self.subtitle_aloud)

        napbutton=Gtk.Button.new_with_label('napiprojekt')
        lf3.pack_start(napbutton,0,0,0)
        napbutton.connect('clicked',self._napiprojekt,None)

        lf3a0=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf3a0,True,True,0)
        lb=Gtk.Label('Słownik')
        lb.set_alignment(1.0,0.5)
        lb.set_size_request(100,0)
        lf3a0.pack_start(lb,0,0,0)
        self.subtitle_dic=Gtk.Entry()
        self.subtitle_dic.set_text(self._prefs.get('diction',''))
        lf3a0.pack_start(self.subtitle_dic,True,True,0)
        self.subtitle_dic.set_icon_from_stock(Gtk.EntryIconPosition.PRIMARY,Gtk.STOCK_CLEAR)
        self.subtitle_dic.connect('icon-press',self._clentry,'diction')
        blbutton=Gtk.Button.new_from_stock(Gtk.STOCK_OPEN)
        blbutton.set_size_request(80,0)
        lf3a0.pack_start(blbutton,0,0,0)
        blbutton.connect('clicked',self._open_dic,self.subtitle_dic)


        lf3a=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf3a,True,True,0)
        lb=Gtk.Label('Autokorekta')
        lb.set_alignment(1.0,0.5)
        lb.set_size_request(100,0)
        lf3a.pack_start(lb,0,0,0)
        self.autoconnect_cb=Gtk.CheckButton.new_with_label('Połącz rozbite kwestie')
        lf3a.pack_start(self.autoconnect_cb,0,0,0)

        
        lf4=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf4,True,True,0)
        lb=Gtk.Label('Wyświetlane')
        lb.set_alignment(1.0,0.5)
        lb.set_size_request(100,0)
        lf4.pack_start(lb,0,0,0)
        
        self.subtitle_displ=Gtk.ComboBoxText.new_with_entry()
        self.subtitle_displ.get_children()[0].set_editable(False)
        lf4.pack_start(self.subtitle_displ,True,True,0)
        dlbutton=Gtk.Button.new_from_stock(Gtk.STOCK_OPEN)
        dlbutton.set_size_request(80,0)
        lf4.pack_start(dlbutton,0,0,0)
        dlbutton.connect('clicked',self._open_subtitles,self.subtitle_displ)
        
        
        mpf=Gtk.Frame.new('mplayer')
        self.mainbox.pack_start(mpf,0,0,0)
        mpfv=Gtk.VBox(0,2)
        mpf.add(mpfv)
        lf5=Gtk.HBox(0,2)
        mpfv.pack_start(lf5,0,0,0)
        lb=Gtk.Label('Audio:')
        lf5.pack_start(lb,0,0,0)
        self.mpl_volcontrol=Gtk.ComboBoxText()
        lb=Gtk.Label('Głośność')
        lf5.pack_start(lb,0,0,0)
        for a in ['Sprzętowa',"Programowa","Programowa x 3","Programowa x 10","Programowa x 30"]:
            self.mpl_volcontrol.append_text(a)
        self.mpl_volcontrol.set_active(self._prefs.get('volctl',1))
        lf5.pack_start(self.mpl_volcontrol,0,0,0)
        self.mpl_volnorm=Gtk.CheckButton.new_with_label('Normalizuj')
        lf5.pack_start(self.mpl_volnorm,0,0,0)
        self.mpl_volnorm.set_active(self._prefs.get('volnorm',False))

        self.mpl_equali=Gtk.CheckButton.new_with_label('Equalizer')
        lf5.pack_start(self.mpl_equali,0,0,0)
        self.mpl_equali.set_active(self._prefs.get('equctl',False))
        self.mpl_equali.connect('toggled',self._equali_onoff,None)
        
        self.frame_equali=Gtk.Frame.new('Equalizer')
        mpfv.pack_start(self.frame_equali,0,0,0)
        ehb=Gtk.HBox()
        self.frame_equali.add(ehb)
        ehg=Gtk.Grid()
        ehb.pack_start(ehg,True,True,20)
        self.equidgets=[]
        ax=self._prefs.get('equval',None)
        if ax:
            try:
                ax=ax.split(':')
                if len(ax) < 10:
                    ax=None
                else:
                    for i,a in enumerate(ax):
                        ax[i]=float(a)
            except:
                ax=None
                
        for i in range(10):
            wg=Gtk.Scale.new_with_range(Gtk.Orientation.VERTICAL,-12.0,12.0,0.1)
            wg.set_digits(1)
            wg.set_inverted(True)
            wg.set_value(ax[i] if ax else 0.0)
            wg.set_size_request(45,100)
            ehg.attach(wg,i,0,1,1)
            self.equidgets.append(wg)
            ehg.attach(Gtk.Label(_EQLABS[i]),i,1,1,1)
        
        
        
        lf50=Gtk.HBox(0,2)
        mpfv.pack_start(lf50,0,0,0)
        lb=Gtk.Label('Video:')
        lf50.pack_start(lb,0,0,0)
        self.deinterlace_cb=Gtk.CheckButton.new_with_label('Deinterlace')
        lf50.pack_start(self.deinterlace_cb,0,0,0)
        self.fullscreen_cb=Gtk.CheckButton.new_with_label('Fullscreen')
        lf50.pack_start(self.fullscreen_cb,0,0,0)
        self.fullscreen_cb.set_active(self._prefs.get('fullscreen',False))
        
        lf5a=Gtk.HBox(0,2)
        mpfv.pack_start(lf5a,0,0,0)
        lb=Gtk.Label('Dodatkowe parametry')
        lf5a.pack_start(lb,0,0,0)
        self.mpl_extras=Gtk.ComboBoxText.new_with_entry()
        lf5a.pack_start(self.mpl_extras,True,True,0)
        self.mpl_extras_dele=Gtk.Button.new_from_stock(Gtk.STOCK_DELETE)
        lf5a.pack_start(self.mpl_extras_dele,0,0,0)
        self.mpl_extras.get_entry().set_icon_from_stock(Gtk.EntryIconPosition.PRIMARY,Gtk.STOCK_CLEAR)
        self.mpl_extras.get_entry().connect('icon-press',self._clentry,None)
        self.mpl_extras.get_entry().connect('changed',self._prextras,None)
        extras=self._prefs.get('extras',None)
        if extras:
            extras=extras.split('\n')
            for a in extras:
                self.mpl_extras.append_text(a)
        self._prextras()
        self.mpl_extras_dele.connect('clicked',self._delextras,None)
        
        

        lf99=Gtk.HBox(0,2)
        self.mainbox.pack_start(lf99,0,0,0)
        but=Gtk.Button.new_with_label('Start')
        but.set_size_request(-1,45)
        lf99.pack_start(but,True,True,40)
        but.connect('clicked',self._start_movie,None);


        
         
        
        self.pilot=Gtk.VBox(0,2)
        frame.add(self.pilot)
        
        lf6=Gtk.HBox(0,2)
        self.pilot.pack_start(lf6,0,0,0)
        lb=Gtk.Label('Głośność')
        lb.set_size_request(100,0)
        lf6.pack_start(lb,0,0,0)

        self._volscale=Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL,0.0,100.0,1.0)
        self._volscale.set_value(100.0)
        lf6.pack_start(self._volscale,True,True,0)
        self._volscale.connect('value-changed',self._setvol,None)

        lf7=Gtk.HBox(0,2)
        self.pilot.pack_start(lf7,0,0,0)
        lb=Gtk.Label('Opóźnienie')
        lb.set_size_request(100,0)
        lf7.pack_start(lb,0,0,0)

        self._delscale=Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL,-10.0,10.0,0.02)
        self._delscale.set_value(0.0)
        lf7.pack_start(self._delscale,True,True,0)
        self._delscale.connect('value-changed',self._setdel,None)

        lf8=Gtk.HBox(0,2)
        self.pilot.pack_start(lf8,0,0,0)
        resyn=Gtk.Button.new_with_label('Synchronizacja')
        lf8.pack_start(resyn,True,True,3)
        resyn.connect('clicked',self._resync,None)
        paubut=Gtk.Button.new_with_label('Pauza')
        lf8.pack_start(paubut,True,True,3)
        paubut.connect('clicked',self._pause,None)


        self.show_all()
        if not self.mpl_equali.get_active():
            self.frame_equali.hide()
        self.pilot.hide()
        self.resize(10,10)




def main():
    player=gui()
    try:
        movie=sys.argv[1]
        if not movie:
            movie=player.selectMovie()
            if not movie:
                exit(0)
        if not os.path.isfile(movie):
            player.alert('Brak pliku\n%s' % movie)
            exit(0)
    except IndexError:
        movie=player.selectMovie()
        if not movie:
            exit(0)
    player.set_movie(movie)
    player.loop()
