#!/usr/bin/env python
# -*- coding: utf-8 -*-


# milena_subconvert.py - Milena Audiobook Creator
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write see:
#               <http://www.gnu.org/licenses/>.

import re,subprocess,sys,getopt,os,os.path


class FormatError(Exception):
	pass

class SubConverter(object):


	class Codec(object):
		unit=0
		joiner='\r\n'
		tester=None
		ident=None
		converter=None

		@staticmethod
		def hms(msec):
			sec=int(msec)/1000
			h=sec/3600
			m=(sec % 3600)/60
			s=sec % 60
			return '%02d:%02d:%02d' % (h,m,s)

		@staticmethod
		def hms2msec(hms):
			hms=hms.split(':')
			return 1000 * (3600 * int(hms[0],10) + 60 * int(hms[1],10) + int(hms[2],10))


		def __init__(self,c):
			self.inphrases=[]
			self.outphrases=[]
			self.converter=c

		def fps(self):
			return self.converter.fps

		def append(self,t1,t2,strs):
			t1+=self.converter.shift
			t2+=self.converter.shift
			if t1 >= 0:
				self.inphrases.append([self.unit,t1,t2,strs])
		
		def feed(self,lines):
			"""
			virtual method
			"""
			raise NotImplementedError('feed')

		def fetch(self):
			"""
			virtual method
			"""
			raise NotImplementedError('fetch')

		def getinfo(self):
			return None

		def setinfo(self):
			return

		def parse(self,txt):
			return txt.split('|')

		def join(self,lines):
			return '|'.join(lines)

		def getunit(self,t):
			# t[0],self.unit,self.fps(),t[1],t[2]
			if t[0] == self.unit:
				return (t[1],t[2])
			if self.unit == 0:
				#konwersja frame to msec
				#print "frame to msec"
				return ((t[1] * 1000.0)/self.fps(),(t[2] * 1000.0)/self.fps())
			else:
				#konwersja msec to frame
				#print "msec to frame"
				return ((t[1] * self.fps())/1000.0,(t[2] * self.fps())/1000.0)

		def getinput(self):
			if len(self.inphrases)>0:
				return self.inphrases.pop(0)
			return None

		def get(self,callab):
			while True:
				a=callab()
				if not a:
					break
				self.outphrases.append(self.emit(a))
			return self.joiner.join(self.outphrases)+'\r\n'

		def finalize(self):
			if len(self.inphrases) == 0:
				return
			for a in range(0,len(self.inphrases)-1):
				phrase=self.inphrases[a]
				if phrase[2] == 0:
					phrase[2]=self.inphrases[a+1][1]
			if self.inphrases[-1][2] == 0:
				self.inphrases[-1][2]=self.inphrases[-1][2]+25*4;

	class SCodec(Codec):
		"""
		klasa bazowa dla MicroDVD i MPL2
		"""
		mpx=1.0

		def feed(self,lines):
			for n,line in enumerate(lines):
				if line == '':
					continue
				r=re.match(self.rx,line)
				if not r:
					raise FormatError(n+1)
				t1=int(r.group(1))
				t2=r.group(2)
				if t2 == '':
					t2=0
				else:
					t2=int(t2)
				t1=t1/self.mpx
				t2=t2/self.mpx
				
				self.append(t1,t2,self.parse(r.group(3)))
			self.finalize()

		def emit(self,phrase):
			#print phrase
			t1,t2=self.getunit(phrase)
			#sys.exit(0)
			return self.lineformat % (int(t1*self.mpx),int(t2*self.mpx),self.join(phrase[3]))

	class mDVD(SCodec):
		"""
		format MicroDVD
		"""
		unit=1
		rx=r'^\{([0-9]+)\}\{([0-9]*)\}\s*(.*)$'
		lineformat='{%d}{%d}%s'
		tester=r'^\{[0-9]+\}\{[0-9]*}'
		ident='d'
		name="MicroDVD"


	class mpl2(SCodec):
		"""
		format MPL2
		"""
		mpx=0.01
		rx=r'^\[([0-9]+)\]\[([0-9]+)\]\s*(.*)$'
		lineformat='[%d][%d]%s'
		tester=r'^\[[0-9]+\]\[[0-9]+]'
		ident='m'
		name="MPL2"

	class TMP(Codec):
		"""
		format TMP - dba o to aby nie nakładać napisów wyjściowych
		"""
		tester=r'^[0-9][0-9]:[0-5][0-9]:[0-5][0-9]:[^0-9]'
		ident='t'
		name="TMPlayer"
		def __init__(self,c):
			super(self.__class__,self).__init__(c)
			self.last_sec=-1;

		def feed(self,lines):
			for n,line in enumerate(lines):
				if line == '':
					continue
				r=re.match(r'([0-9][0-9]:[0-5][0-9]:[0-5][0-9]):(.*)$',line)
				if not r:
					raise FormatError(n+1)
				t1=self.hms2msec(r.group(1))
				t2=0
				self.append(t1,t2,self.parse(r.group(2)))
			self.finalize()

		def emit(self,phrase):
			t1,t2=self.getunit(phrase)
			sec=t1/1000
			if sec <= self.last_sec:
				sec=self.last_sec+1
				t1=sec*1000
			self.last_sec=sec
			return '%s:%s' % (self.hms(t1),self.join(phrase[3]))


	class SubRip(Codec):
		"""
		SubRip
		"""
		joiner='\r\n\r\n'
		ident='s'
		name="SubRip"
		tester=r'([0-9]+\s+)?[0-9]+:[0-5][0-9]:[0-5][0-9],[0-9]+\s+-->\s+[0-9]+:[0-5][0-9]:[0-5][0-9],[0-9]+$'

		def __init__(self,c):
			super(self.__class__,self).__init__(c)
			self.outcounter=0

		def emit(self,phrase):
			self.outcounter+=1
			t1,t2=self.getunit(phrase)
			p='%d\r\n%s,%03d --> %s,%03d\r\n' % (
				self.outcounter,
				self.hms(t1),t1 % 1000,
				self.hms(t2),t2 % 1000)
			p+='\r\n'.join(phrase[3])
			return p

		def feed(self,lines):
			def amsec(x):
				x=x.split(',')
				return self.hms2msec(x[0])+int(x[1],10)
			state=0
			t1=None
			t2=None
			query=None
			for n,line in enumerate(lines):
				if line == '':
					if state == 2:
						self.append(t1,t2,query)
					state=0
					continue
				if state == 0:
					if re.match('^[0-9]+$',line):
						state = 1
						continue
					r=re.match(r'[0-9]+\s+([0-9]+:[0-5][0-9]:[0-5][0-9],[0-9]+)\s+-->\s+([0-9]+:[0-5][0-9]:[0-5][0-9],[0-9]+)$',line)
					if r:
						t1=amsec(r.group(1))
						t2=amsec(r.group(2))
						query=[]
						state = 2
						continue
					raise FormatError(n+1)
				elif state == 1:
					r=re.match(r'([0-9]+:[0-5][0-9]:[0-5][0-9],[0-9]+)\s+-->\s+([0-9]+:[0-5][0-9]:[0-5][0-9],[0-9]+)$',line)
					if r:
						t1=amsec(r.group(1))
						t2=amsec(r.group(2))
						query=[]
						state = 2
						continue
					raise FormatError(n+1)
				else:
					query.append(line)
			if state == 2:
				self.append(t1,t2,query)

	class FAB(Codec):
		joiner='\r\n\r\n'
		ident='f'
		name="FAB"
		def __init__(self,c):
			super(self.__class__,self).__init__(c)
			self.outcounter=0

		def emit(self,phrase):
			self.outcounter+=1
			t1,t2=self.getunit(phrase)
			p='%04d : %s:%02d  %s:%02d\r\n' % (
				self.outcounter,
				self.hms(t1),(t1 % 1000)/10,
				self.hms(t2),(t2 % 1000)/10)
			p+='\r\n'.join(phrase[3])
			return p


	class Jaco(Codec):
		"""
		Format JacoSUB - wyłącznie koder
		"""
		ident='j'
		name="JacoSUB"

		def ref(self,x):
			if x.startswith('/'):
				x='\\I'+x[1:]
			x=x.replace('~','\\~')
			x=re.sub(r'\{y:i\}','\\I',x,0,re.I)
			x=re.sub(r'\{y:b\}','\\B',x,0,re.I)
			x=re.sub(r'\{.*?\}','',x,0,re.I)
			return x
		def emit(self,phrase):
			t1,t2=self.getunit(phrase)
			return '%s.%02d %s.%02d {~} %s' % (
				self.hms(t1),(t1 % 1000)/100,
				self.hms(t2),(t2 % 1000)/100,
				'\\n'.join(self.ref(x) for x in phrase[3]))

		def get(self,callab):
			return "#TIMERES 100\r\n"+super(self.__class__,self).get(callab)


	class ASS(Codec):
		"""
		Format SSA/ASS
		"""
		ident="ass"
		name="SSA/ASS"
		tester=r"(?:...)?\[Script Info\]"
		
		def _hmt(self,isa):
			mu=isa.split('.')
			hms=mu[0].split(':')
			return 1000 * (3600 * int(hms[0],10) + 60 * int(hms[1],10) + int(hms[2],10)) + int(mu[1],10) * 10
		
		def feed(self,lines):
			state=0
			t1=None
			tp1=None
			t2=None
			tp2=None
			fmat=None
			
			for n,line in enumerate(lines):
				if line == '':
					continue
				if line.startswith('['):
					if line.lower().startswith('[events]'):
						state=1
					else:
						state=0
					continue
				if state == 0:
					continue
				r=re.match(r'^format:\s+(.*)$',line,re.I)
				if r:
					line=r.group(1).split(',')
					fmat=len(line)-1
					for i,a in enumerate(line):
						a=a.lower().strip()
						if a=='start':
							tp1=i
						elif a=='end':
							tp2=i
					if tp1 is None:
						raise Exception('Bad format')
					if tp2 is None:
						tp2=tp1
					state=2
					continue
				if state != 2:
					continue
				line=re.sub(r'\{.*?\}','',line)
				line=line.split(',',fmat)
				if len(line) < fmat:
					continue
				t1=self._hmt(line[tp1])
				t2=self._hmt(line[tp2])
				query=line[fmat].replace(r'\n',' ').replace(r'\N',' ')
				self.append(t1,t2,[query])
			self.inphrases.sort(cmp=lambda x,y: x[1]-y[1])
			
				
					
	
	class SubV(Codec):
		"""
		Format SubViewer
		"""
		ident='sv'
		joiner='\r\n\r\n'
		vj='\r\n'
		name="SubViewer"
		tester=r'^[0-9]?[0-9]:[0-5]?[0-9]:[0-5]?[0-9]:[0-9][0-9],[0-9]?[0-9]:[0-5]?[0-9]:[0-5]?[0-9]:[0-9][0-9]$'

		def emit(self,phrase):
			t1,t2=self.getunit(phrase)
			return '%s:%02d,%s:%02d\r\n%s' % (
				self.hms(t1),(t1 % 1000)/10,
				self.hms(t2),(t2 % 1000)/10,
				self.vj.join(phrase[3]))

		def feed(self,lines):
			state=0
			t1=None
			t2=None
			query=None
			for n,line in enumerate(lines):
				if state == 0:
					if line == '' or line.startswith('['):
						continue
					state=1
				if line == '':
					if state == 2:
						self.append(t1,t2,query)
					state=1
					continue
				if state == 1:
					r=re.match(r'(^[0-9]?[0-9]:[0-5]?[0-9]:[0-5][0-9]):([0-9][0-9]),([0-9]?[0-9]:[0-5]?[0-9]:[0-5][0-9]):([0-9][0-9])$',line)
					if not r:
						raise FormatError(n+1)
					t1=self.hms2msec(r.group(1))+10*int(r.group(2),10)
					t2=self.hms2msec(r.group(3))+10*int(r.group(4),10)
					query=[]
					state=2
					continue
				li=line.split('[br]')
				query = query + li
			if state == 2:
				self.append(t1,t2,query)




	class SubV2(SubV):
		ident='sv2';
		name="SubViewer2"
		joiner='\r\n\r\n'
		vj='[br]'
		tester=None


	codecs=[mDVD,mpl2,FAB,Jaco,SubRip,TMP,SubV,SubV2,ASS]
	hstring="""Sposób użycia
	%s opcje
gdzie opcje to:
	-m film.avi
	-E rozszerzenia do automatycznego rozpoznawania napisów oddzielone
		przecinkami (domyślnie sub,srt,txt)
	-t plik napisów w jednym z formatów:
	   ###
	-f fps
	-s przesunięcie w czasie, liczba całkowita i opcjonalna jednostka
	   s (sekundy), ms (milisekundy) lub f (ramki). Domyślnie ms.
	-F format pliku wynikowego:
$$$
	-e kodowanie pliku wynikowego (domyślnie bez przekodowania)
	   Nie należy stosować innych niż CP1250, ISO-8859-2 i UTF-8.
	-o plik wynikowy
	-O rozszerzenie do automatycznego tworzenia pliku wynikowego
	-y pozwól na nadpisanie pliku wynikowego
Ścieżka do pliku filmu musi być podana, jeśli:
	automatycznie szukamy pliku napisów, lub
	automatycznie tworzymy nazwę pliku wynikowego, lub
	konwertujemy między formatami ramkowym i czasowym, a nie podajemy fps
""" % sys.argv[0]

	clines=[]
	ccodes={}
	incods=[]
	for codec in codecs:
		ccodes[codec.ident]=codec
		clines.append('\t   %3s - %s' % (codec.ident,codec.name))
		if codec.tester:
			incods.append(codec.name)
	hstring=hstring.replace('$$$','\n'.join(clines)).\
				replace('###',', '.join(incods))




	def __init__(self):
		self.movie=None
		self.inname=None
		self.outname=None
		self.inext=['sub','srt','txt','ass']
		self.outext=None
		self.fps=None
		self.format='m'
		self.incodec=None
		self.outcodec=None
		self.overwrite=False
		self.mbase=None
		self.infile=None
		self.recode=None
		self.shift=0
		self.shunit=None
		self.gui=None


	@staticmethod
	def recode_file(lines,encoding,lang='pl'):
		if isinstance(lines,list):
			lines='\r\n'.join(lines)+'\r\n'
		try:
			enca=subprocess.Popen(['enca','-L',lang,'-x',encoding],
				stdin=subprocess.PIPE,
				stdout=subprocess.PIPE,
				stderr=subprocess.PIPE)
			rc=enca.communicate(lines)
			rc=rc[0]
		except:
			return None
		if not rc:
			return None
		return rc

	def usage(self,txt=None):
		if txt:
			sys.stderr.write(txt+'\n')
		sys.stderr.write(self.hstring)
		sys.exit(1)

	def get_mbase(self):
		if not self.mbase:
			dy=os.path.dirname(self.movie)
			ba=os.path.basename(self.movie)
			n=ba.rfind('.')
			if n < 1:
				self.mbase=self.movie
			else:
				ba=ba[:n]
				self.mbase=os.path.join(dy,ba)

	def getrate(self):

		def cpute(txt):
			a=txt.split('/')
			if len(a) == 2:
				return float(a[0])/float(a[1])
			return float(a)

		if self.fps is not None:
			return None
		if not self.movie:
			return "Podaj ścieżkę do filmu lub fps"
		p=subprocess.Popen(['ffprobe','-show_streams',self.movie],
			stderr=subprocess.PIPE,
			stdout=subprocess.PIPE)
		ostr=p.communicate()[0]
		r=re.match(r'.*\[STREAM\](.*\ncodec_type=video\n.*?)\[/STREAM\](.*)$',ostr,re.DOTALL)
		if r:
			g=r.group(1)
			r=re.match(r'.*\nr_frame_rate=(.*?)\n',g,re.DOTALL)
			if r and r.group(1) != '0/0':
				self.fps=cpute(r.group(1))
				return None
			r=re.match(r'.*\ntime_base=(.*?)\n',g,re.DOTALL)
			if r and r.group(1) != '0/0':
				self.fps=1.0/cpute(r.group(1))
				return None
			r=re.match(r'.*\nduration=(.*?)\n',g,re.DOTALL)
			if r:
				du=float(r.group(1))
				r=re.match(r'.*\nnb_frames=(.*?)\n',g,re.DOTALL)
				if r:
					self.fps=float(r.group(1))/du
					return None
		return "Nie mogę określić FPS"

	def read_infile(self,fname):
		try:
			lines=open(fname,'r').read()
			if self.recode:
				lines=self.recode_file(lines,self.recode)
				if not lines:
					return "Enca nie zwraca wyniku"
			lines=lines.replace('\r\n','\n').split('\n')
		except:
			return "Nie mogę otworzyć pliku %s" % fname
		codec=None
		for n,line in enumerate(lines):
			if n >= 100:
				break
			line=line.strip()
			for a in self.codecs:
				if a.tester and re.match(a.tester,line):
					codec=a(self)
					break
			if codec:
				break
			
		if not codec:
			return "Plik %s nie jest plikiem napisów" % fname
		self.incodec=codec
		for n,line in enumerate(lines):
			lines[n]=line.strip()
		self.infile=lines
		return None



	def find_subtitles(self):
		if self.inname:
			if not os.path.exists(self.inname):
				return "Plik napisów %s nie istnieje" % self.inname
			else:
				return self.read_infile(self.inname)
		if not self.movie:
			return "Podaj plik z filmem jeśli nie podajes pliku napisów"
		self.get_mbase()
		for ext in self.inext:
			name=self.mbase+'.'+ext
			if os.path.exists(name):
				if self.read_infile(name):
					continue
				self.inname=name
				break
		if not self.inname:
			return "Nie mogę dopasować pliku napisów"
		return None

	def find_outfile(self):
		if self.outname == '-':
			return None
		if not self.outname:
			if not self.movie or not self.outext:
				return "Podaj nazwę filmu i rozszerzenie jeśli nie podajesz pliku wynikowego"
			self.get_mbase()
			self.outname=self.mbase+'.'+self.outext
		if not self.gui and not self.overwrite and os.path.exists(self.outname):
			return "Plik wynikowy %s istnieje, użyj -y aby go nadpisać" % self.outname
		return None


	def init_with_args(self,args):
		global ccodes
		try:
			opts,argv=getopt.getopt(sys.argv[1:],"hm:E:t:f:F:o:O:ye:s:")
		except getopt.GetoptError, err:
			self.usage(str(err))
		for opt,arg in opts:
			if opt == '-m':
				self.movie=arg
			elif opt == '-E':
				self.inext=arg.split(',')
			elif opt == '-t':
				self.inname=arg
			elif opt == '-f':
				self.fps=float(arg.replace(',','.'))
			elif opt == '-F':
				if arg not in self.ccodes:
					self.usage("Nieznany kodek %s" % arg)
				self.format=arg
			elif opt == '-o':
				self.outname=arg
			elif opt == '-O':
				self.outext=arg
			elif opt == '-e':
				self.recode=arg
			elif opt == '-s':
				r = re.match(r'^(-?[0-9]+)(s|ms|f|)$',arg)
				if not r:
					self.usage("Błędny parametr dla s")
				u=r.group(2)
				n=int(r.group(1),10)
				if u == 's':
					n = n * 1000
				self.shift = n
				self.shunit = 1 if u == 'f' else 0
			elif opt == '-y':
				self.overwrite=True
			else:
				self.usage()

		a=self.find_subtitles()
		if a:
			self.usage(a)
		a=self.find_outfile()
		if a:
			self.usage(a)
		self.outcodec=self.ccodes[self.format](self)
		if self.outcodec.unit != self.incodec.unit or \
				(self.shift and self.shunit != self.incodec.unit):
			a=self.getrate()
			if a:
				self.usage(a)

			if self.shift and self.shunit != self.incodec.unit:
				if self.incodec.unit == 1:
					self.shift=(self.shift * self.fps) / 1000.0
				else:
					self.shift=(self.shift * 1000.0)/self.fps


	def run(self):
		self.incodec.feed(self.infile)
		info=self.incodec.getinfo()
		if info:
			self.outcodec.setinfo(info)
		return self.outcodec.get(self.incodec.getinput)
		
	def simplify(self,inname,outname):
		self.inname=inname
		self.recode='utf-8'
		rc=self.find_subtitles()
		if rc:
			return rc
		self.incodec.feed(self.infile)
		if self.incodec.unit == 1:
			# przyjmijmy bezpiecznie 24 fps
			syllable=4
		else:
			syllable=166
		for a in self.incodec.inphrases:
			if len(a[3])>1:
				a[3]=[' '.join(a[3])]
			a[3][0]=self.simplify_line(a[3][0])
		for i in range(len(self.incodec.inphrases)-1,0,-1):
			s2=self.incodec.inphrases[i][3][0].decode('utf-8')
			s1=self.incodec.inphrases[i-1][3][0].decode('utf-8')
			if len(s1)<30:
				continue
			if len(s2)<5:
				continue
			link=False
			if s1.endswith(u','):
				if s2[0].isalnum():
					link=True
			elif s1[-1].islower():
				if s2[0].islower():
					link=True
			if not link:
				continue
			lc=self.count_syllables(s1)*syllable*1.8
			#print lc, self.incodec.inphrases[i][1] - (self.incodec.inphrases[i-1][1]+lc)
			if self.incodec.inphrases[i-1][1]+lc >= self.incodec.inphrases[i][1]:
				self.incodec.inphrases[i-1][2]=self.incodec.inphrases[i][2]
				#print "Connect",self.incodec.inphrases[i-1][1]
				#print " --- ",self.incodec.inphrases[i-1][3][0]
				#print " +++ ",self.incodec.inphrases[i][3][0]
				self.incodec.inphrases[i-1][3][0] += ' '+self.incodec.inphrases[i][3][0]
				self.incodec.inphrases.pop(i)
		self.outcodec=self.mDVD(self) if self.incodec.unit == 1 else self.mpl2(self)
		s=self.outcodec.get(self.incodec.getinput)
		try:
			f=open(outname,'w')
		except:
			return "Nie mogę otworzyć pliku do zapisu"
		f.write(s)
		f.close()
		return None

	def count_syllables(self,line):
		if isinstance(line,str):
			line=line.decode('utf-8')
		line=line.lower()
		line=re.sub(ur'(^|[i\s])au',u'a',line,0,re.UNICODE)
		line=re.sub(ur'i[aeiouąęó]',u'a',line,0,re.UNICODE)
		line=re.sub(ur'[^aeiouyąęó]','',line,0,re.UNICODE)
		return len(line)
	
	def simplify_line(self,line):
		line=re.sub(r'\{.*?\}','',line)
		line=re.sub(r'\\.','',line)
		return line

def main():
	a=SubConverter()
	if len(sys.argv) > 1:
		a.init_with_args(sys.argv)
	else:
		a.usage("=== Wersja GUI w przygotowaniu!!! ===")
	txt=a.run()
	if a.outname == '-':
		sys.stdout.write(txt)
	else:
		open(a.outname,'w').write(txt)

if __name__ == "__main__":
	main()

